<?php

namespace Litepci\Project\Policies;

use Litepie\User\Contracts\UserPolicy;
use Litepci\Project\Models\Milestone;

class MilestonePolicy
{

    /**
     * Determine if the given user can view the milestone.
     *
     * @param UserPolicy $user
     * @param Milestone $milestone
     *
     * @return bool
     */
    public function view(UserPolicy $user, Milestone $milestone)
    {
        if ($user->canDo('project.milestone.view') && $user->isAdmin()) {
            return true;
        }

        return $milestone->user_id == user_id() && $milestone->user_type == user_type();
    }

    /**
     * Determine if the given user can create a milestone.
     *
     * @param UserPolicy $user
     * @param Milestone $milestone
     *
     * @return bool
     */
    public function create(UserPolicy $user)
    {
        return  $user->canDo('project.milestone.create');
    }

    /**
     * Determine if the given user can update the given milestone.
     *
     * @param UserPolicy $user
     * @param Milestone $milestone
     *
     * @return bool
     */
    public function update(UserPolicy $user, Milestone $milestone)
    {
        if ($user->canDo('project.milestone.edit') && $user->isAdmin()) {
            return true;
        }

        return $milestone->user_id == user_id() && $milestone->user_type == user_type();
    }

    /**
     * Determine if the given user can delete the given milestone.
     *
     * @param UserPolicy $user
     * @param Milestone $milestone
     *
     * @return bool
     */
    public function destroy(UserPolicy $user, Milestone $milestone)
    {
        return $milestone->user_id == user_id() && $milestone->user_type == user_type();
    }

    /**
     * Determine if the given user can verify the given milestone.
     *
     * @param UserPolicy $user
     * @param Milestone $milestone
     *
     * @return bool
     */
    public function verify(UserPolicy $user, Milestone $milestone)
    {
        if ($user->canDo('project.milestone.verify')) {
            return true;
        }

        return false;
    }

    /**
     * Determine if the given user can approve the given milestone.
     *
     * @param UserPolicy $user
     * @param Milestone $milestone
     *
     * @return bool
     */
    public function approve(UserPolicy $user, Milestone $milestone)
    {
        if ($user->canDo('project.milestone.approve')) {
            return true;
        }

        return false;
    }

    /**
     * Determine if the user can perform a given action ve.
     *
     * @param [type] $user    [description]
     * @param [type] $ability [description]
     *
     * @return [type] [description]
     */
    public function before($user, $ability)
    {
        if ($user->isSuperuser()) {
            return true;
        }
    }
}
